/**
 * @file  Channel4ModuleHardware.cpp
 * @brief Implementierung der Klasse CChannel4ModuleHardware.
 */
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Channel4ModuleHardware.h"
#include <stdio.h>

// ##################################################################
/**
 * @brief Initialisieren der Membervariablen.
 *
 * @param id Nummer der Kanalklasse fuer z.B. Trace.
 */
void CChannel4ModuleHardware::InitClass(short channelID, short deviceID)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::InitClass [ %i, %i ]\n", __FILE__, __LINE__, channelID, deviceID);

}

// ##################################################################
/**
 * @brief Destruktor.
 *
 */
CChannel4ModuleHardware::~CChannel4ModuleHardware() 
{
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel4ModuleHardware::GetErrorModule(short module, short error, char *text)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetErrorModule [ %i, %i, 0x%08X ]\n", __FILE__, __LINE__, module, error, text);

	// Ist IV_MODULE_HARDWARE ?
	if (module == IV_MODULE_HARDWARE)
	{

	}

	// Ermittlung des Fehlertextes an Basisklasse abgeben.
	CChannel4Protocol::GetErrorModule(module, error, text);
}


// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel4ModuleHardware::GetErrorFunction(short module, short func, short error, char *text)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetErrorFunction [ %i, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, module, func, error, text);

	if (module == IV_MODULE_HARDWARE)
	{
		// Sonderfehler fr Passwortschutz abhandeln.
		if (error == 28)
		{
			// Die Modulfunktionen liefern den Fehlercode 28, wenn kein oder ein falsches Passwort verwendet wurde.
			// Es ist in diesem Fall zuerst das richtige Passwort fr die Kommunikation erforderlich.
			sprintf(text, "( %i ): Korrektes Passwort fr die Kommunikation erforderlich.", error);
			return;
		}
		else if (error == 29)
		{
			// Die Modulfunktionen liefern den Fehlercode 29, wenn 3 mal ein falsches Passwort verwendet wurde.
			// Die Befehlsausfhrung ist danach fr 60 Sekunden blockiert.
			sprintf(text, "( %i ): Gert ist aufgrund von fehlerhaften Passworteingaben fr 60 Sekunden gesperrt.", error);
			return;
		}

		switch (func)
		{
		case IV_FUNCTION_PING:
		// Setzen/Ermitteln des Geraetetyp
		case IV_FUNCTION_SET_TYPE_DEVICE:
		case IV_FUNCTION_GET_TYPE_DEVICE:

		// Setzen/Ermitteln des Boardtyp (Redesignversion z.B.: 0x13) (in Nibbles)
		case IV_FUNCTION_SET_TYPE_BOARD:
		case IV_FUNCTION_GET_TYPE_BOARD:

		// Setzen/Ermitteln der Boardkomponente: RS485 fuer Kommunikation mit PC
		case IV_FUNCTION_SET_COMPONENT_RS485_0:
		case IV_FUNCTION_GET_COMPONENT_RS485_0:

		// Setzen/Ermitteln der Boardkomponente: TCP/IP
		case IV_FUNCTION_SET_TYPE_TCPIP:
		case IV_FUNCTION_GET_TYPE_TCPIP:

		// Setzen/Ermitteln der Boardkomponente: Modem MC35i/MC55
		case IV_FUNCTION_SET_TYPE_MODEM_MC:
		case IV_FUNCTION_GET_TYPE_MODEM_MC:

		// Setzen/Ermitteln der Boardkomponente: RS485 fuer Zutrittskontrolle
		case IV_FUNCTION_SET_COMPONENT_RS485_1:
		case IV_FUNCTION_GET_COMPONENT_RS485_1:

		// Setzen/Ermitteln des Transpondertyps
		case IV_FUNCTION_SET_TYPE_TRANSPONDER:
		case IV_FUNCTION_GET_TYPE_TRANSPONDER:

		// Setzen/Ermitteln des Displaytyp
		case IV_FUNCTION_SET_TYPE_DISPLAY:
		case IV_FUNCTION_GET_TYPE_DISPLAY:

		// Setzen/Ermitteln des Tastaturtyps
		case IV_FUNCTION_SET_TYPE_KEYPAD:
		case IV_FUNCTION_GET_TYPE_KEYPAD:

		// Setzen/Ermitteln der Multimediakarte
		case IV_FUNCTION_SET_TYPE_MMC:
		case IV_FUNCTION_GET_TYPE_MMC:

		// Setzen/Ermitteln des GPS - Typs
		case IV_FUNCTION_SET_TYPE_GPS:
		case IV_FUNCTION_GET_TYPE_GPS:

		// Setzen/Ermitteln der USB - Komponente
		case IV_FUNCTION_SET_COMPONENT_USB:
		case IV_FUNCTION_GET_COMPONENT_USB:

		// Setzen/Ermitteln der Hardwareconfigurationsnummer
		case IV_FUNCTION_SET_HW_CONFIG:
		case IV_FUNCTION_GET_HW_CONFIG:

		// Setzen/Ermitteln des Funkmodultyps
		case IV_FUNCTION_SET_TYPE_RADIO:
		case IV_FUNCTION_GET_TYPE_RADIO:

		// Setzen/Ermitteln des Funkuhrmodultyps
		case IV_FUNCTION_SET_TYPE_DCF77:
		case IV_FUNCTION_GET_TYPE_DCF77:

		// Setzen/Ermitteln der analogen Eingaenge - Komponente
		case IV_FUNCTION_SET_COMPONENT_IN_ANALOG:
		case IV_FUNCTION_GET_COMPONENT_IN_ANALOG:

		// Setzen/Ermitteln der digitalen Eingaenge - Komponente
		case IV_FUNCTION_SET_COMPONENT_IN_DIGITAL:
		case IV_FUNCTION_GET_COMPONENT_IN_DIGITAL:

		// Setzen/Ermitteln der digitalen Ausgaenge - Komponente
		case IV_FUNCTION_SET_COMPONENT_OUT_DIGITAL:
		case IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL:

		// Setzen/Ermitteln des Fingerprint + Typs
		case IV_FUNCTION_SET_TYPE_FINGERPRINT:
		case IV_FUNCTION_GET_TYPE_FINGERPRINT:

		case IV_FUNCTION_SET_TYPE_EXTERNAL_READER:
		case IV_FUNCTION_GET_TYPE_EXTERNAL_READER:

		// Optionen schreiben/lesen
		case IV_FUNCTION_SET_OPTION_IOMODULE:
		case IV_FUNCTION_GET_OPTION_IOMODULE:

		case IV_FUNCTION_SET_OPTION_COMSERVER:
		case IV_FUNCTION_GET_OPTION_COMSERVER:

		case IV_FUNCTION_SET_OPTION_TBADAPTION:
		case IV_FUNCTION_GET_OPTION_TBADAPTION:
			sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error);
			break;

		default: sprintf(text, "( %i ): Nicht eingetragener Fehlerblock fr Funktion!", error); break;
		}
		
		return;
	}

	// Ermittlung des Fehlertextes an Basisklasse abgeben.
	CChannel4Protocol::GetErrorFunction(module, func, error, text);
}

//#########################################################
//###################### HARDWARE #########################
//#########################################################

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleHardware::Call(short module, short func, unsigned char *data, int &count)
{
	if (module == IV_MODULE_HARDWARE)
	{
		switch (func)
		{
		case IV_FUNCTION_PING: return Ping(GetDeviceID(), data, count);

		// Setzen/Ermitteln des Geraetetyp
		case IV_FUNCTION_SET_TYPE_DEVICE: return SetTypeDevice(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_TYPE_DEVICE: return GetTypeDevice(GetDeviceID(), (unsigned char&)data[0]); 

		// Setzen/Ermitteln des Boardtyp (Redesignversion z.B.: 0x13) (in Nibbles)
		case IV_FUNCTION_SET_TYPE_BOARD: return SetTypeBoard(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_TYPE_BOARD: return GetTypeBoard(GetDeviceID(), (unsigned char&)data[0]);

		// Setzen/Ermitteln der Boardkomponente: RS485 fuer Kommunikation mit PC
		case IV_FUNCTION_SET_COMPONENT_RS485_0: return SetComponentRS485_0(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_COMPONENT_RS485_0: return GetComponentRS485_0(GetDeviceID(), (unsigned char&)data[0]);

		// Setzen/Ermitteln der Boardkomponente: TCP/IP
		case IV_FUNCTION_SET_TYPE_TCPIP: return SetTypeTCPIP(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_TYPE_TCPIP: return GetTypeTCPIP(GetDeviceID(), (unsigned char&)data[0]); 

		// Setzen/Ermitteln der Boardkomponente: Modem MC35i/MC55
		case IV_FUNCTION_SET_TYPE_MODEM_MC: return SetTypeModemMC(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_TYPE_MODEM_MC: return GetTypeModemMC(GetDeviceID(), (unsigned char&)data[0]); 

		// Setzen/Ermitteln der Boardkomponente: RS485 fuer Zutrittskontrolle
		case IV_FUNCTION_SET_COMPONENT_RS485_1: return SetComponentRS485_1(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_COMPONENT_RS485_1: return GetComponentRS485_1(GetDeviceID(), (unsigned char&)data[0]); 

		// Setzen/Ermitteln des Transpondertyps
		case IV_FUNCTION_SET_TYPE_TRANSPONDER: return SetTypeTransponder(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_TYPE_TRANSPONDER: return GetTypeTransponder(GetDeviceID(), (unsigned char&)data[0]); 

		// Setzen/Ermitteln des Displaytyp
		case IV_FUNCTION_SET_TYPE_DISPLAY: return SetTypeDisplay(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_TYPE_DISPLAY: return GetTypeDisplay(GetDeviceID(), (unsigned char&)data[0]); 

		// Setzen/Ermitteln des Tastaturtyps
		case IV_FUNCTION_SET_TYPE_KEYPAD: return SetTypeKeypad(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_TYPE_KEYPAD: return GetTypeKeypad(GetDeviceID(), (unsigned char&)data[0]); 

		// Setzen/Ermitteln der Multimediakarte
		case IV_FUNCTION_SET_TYPE_MMC: return SetTypeMMC(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_TYPE_MMC: return GetTypeMMC(GetDeviceID(), (unsigned char&)data[0]); 

		// Setzen/Ermitteln des GPS - Typs
		case IV_FUNCTION_SET_TYPE_GPS: return SetTypeGPS(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_TYPE_GPS: return GetTypeGPS(GetDeviceID(), (unsigned char&)data[0]); 

		// Setzen/Ermitteln der USB - Komponente
		case IV_FUNCTION_SET_COMPONENT_USB: return SetComponentUSB(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_COMPONENT_USB: return GetComponentUSB(GetDeviceID(), (unsigned char&)data[0]); 

		// Setzen/Ermitteln der Hardwareconfigurationsnummer
		case IV_FUNCTION_SET_HW_CONFIG: return SetHWConfig(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_HW_CONFIG: return GetHWConfig(GetDeviceID(), (unsigned char&)data[0]); 

		// Setzen/Ermitteln des Funkmodultyps
		case IV_FUNCTION_SET_TYPE_RADIO: return SetTypeRadio(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_TYPE_RADIO: return GetTypeRadio(GetDeviceID(), (unsigned char&)data[0]); 

		// Setzen/Ermitteln des Funkuhrmodultyps
		case IV_FUNCTION_SET_TYPE_DCF77: return SetTypeDCF77(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_TYPE_DCF77: return GetTypeDCF77(GetDeviceID(), (unsigned char&)data[0]); 

		// Setzen/Ermitteln der analogen Eingaenge - Komponente
		case IV_FUNCTION_SET_COMPONENT_IN_ANALOG: return SetComponentInAnalog(GetDeviceID(), *(unsigned int*)data, *(unsigned short*)&data[4]);
		case IV_FUNCTION_GET_COMPONENT_IN_ANALOG: return GetComponentInAnalog(GetDeviceID(), (unsigned int&)data[0]); 

		// Setzen/Ermitteln der digitalen Eingaenge - Komponente
		case IV_FUNCTION_SET_COMPONENT_IN_DIGITAL: return SetComponentInDigital(GetDeviceID(), *(unsigned int*)data, *(unsigned short*)&data[4]);
		case IV_FUNCTION_GET_COMPONENT_IN_DIGITAL: return GetComponentInDigital(GetDeviceID(), (unsigned int&)data[0]); 

		// Setzen/Ermitteln der digitalen Ausgaenge - Komponente
		case IV_FUNCTION_SET_COMPONENT_OUT_DIGITAL: return SetComponentOutDigital(GetDeviceID(), *(unsigned int*)data, *(unsigned short*)&data[4]);
		case IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL: return GetComponentOutDigital(GetDeviceID(), (unsigned int&)data[0]);

		// Setzen/Ermitteln des Fingerprint + Typs
		case IV_FUNCTION_SET_TYPE_FINGERPRINT: return SetTypeFingerprint(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_TYPE_FINGERPRINT: return GetTypeFingerprint(GetDeviceID(), (unsigned char&)data[0]);

		case IV_FUNCTION_SET_TYPE_EXTERNAL_READER: return SetTypeExternalReader(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_TYPE_EXTERNAL_READER: return GetTypeExternalReader(GetDeviceID(), (unsigned char&)data[0]);

		// Optionen schreiben/lesen
		case IV_FUNCTION_SET_OPTION_IOMODULE: return SetOptionIOModule(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_OPTION_IOMODULE: return GetOptionIOModule(GetDeviceID(), (unsigned char&)data[0]);

		case IV_FUNCTION_SET_OPTION_COMSERVER: return SetOptionComServer(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_OPTION_COMSERVER: return GetOptionComServer(GetDeviceID(), (unsigned char&)data[0]);

		case IV_FUNCTION_SET_OPTION_TBADAPTION: return SetOptionTBAdaption(GetDeviceID(), data[0], *(unsigned short*)&data[1]);
		case IV_FUNCTION_GET_OPTION_TBADAPTION: return GetOptionTBAdaption(GetDeviceID(), (unsigned char&)data[0]);

		// Kommando fuer Pruefplatz
		case IV_FUNCTION_INSPECTION_STATION_OK: return CommandInspectionStationOK(GetDeviceID());

		// Funktion nicht definiert
		default: return 0x0200;
		}
	}

	return CChannel4Protocol::Call(module, func, data, count);
}

// ##################################################################
/**
 * @brief Prueft die Erreichbarkeit des Kommunikationspartners.
 *
 * @param deviceID Anzusprechender Kommunikationspartner.
 */
int CChannel4ModuleHardware::Ping(short deviceID, unsigned char *data, int count)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::Ping [ %i ]\n", __FILE__, __LINE__, deviceID);

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		ResetError();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Zu sendende Daten kopieren?
		if (data && count > 0)
		{
			memcpy(m_data, data, count);
		}
		else
		{
			count = 0;
		}

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_PING, m_data, count, 2)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_PING);
			return m_result;
		}
	}
	else
	{
		int result;

		// Parameterdaten werden beibehalten

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_PING, data, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::Ping [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}
	
	return m_result;
}

// ##################################################################
/**
 * @brief Ermitteln des Geraetetyps.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Geraetetyp
 *             0 = PZE - Master IV
 *             1 = Zutritts - Master IV
 *             2 = BDE - Master IV
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeDevice(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetTypeDevice [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DEVICE, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DEVICE);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetTypeDevice [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DEVICE, &m_hwInfo.typeDevice, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetTypeDevice [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt die vorliegende Board-Revisionsnummer.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Revisionsnummer in der Form (HiNibble Majornummer).(LoNibble Minornummer)
 *             z. B. 0x14 -> Major 1, Minor (Redesign Nr.) 4
 */
int CChannel4ModuleHardware::GetTypeBoard(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetTypeBoard [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_BOARD, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_BOARD);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetTypeBoard [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_BOARD, &m_hwInfo.typeBoard, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetTypeBoard [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetComponentRS485_0(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetComponentRS485_0 [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_0, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_0);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetComponentRS485_0 [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_0, &m_hwInfo.componentRS485_0, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetComponentRS485_0 [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Ermittelte Variante.
 *              0 = Keines
 *              1 = MC35i
 *              2 = MC55
 *            ... = ...
 */
int CChannel4ModuleHardware::GetTypeTCPIP(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetTypeTCPIP [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TCPIP, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TCPIP);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetTypeTCPIP [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TCPIP, &m_hwInfo.typeTCPIP, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetTypeTCPIP [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Ermittelte Variante.
 *              0 = Keines
 *              1 = MC35i
 *              2 = MC55
 *            ... = ...
 */
int CChannel4ModuleHardware::GetTypeModemMC(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetTypeModemMC [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MODEM_MC, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MODEM_MC);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetTypeModemMC [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MODEM_MC, &m_hwInfo.typeModemMC, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetTypeModemMC [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetComponentRS485_1(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetComponentRS485_1 [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_1, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_1);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetComponentRS485_1 [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_1, &m_hwInfo.componentRS485_1, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetComponentRS485_1 [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Ermittelte Variante.
 *             0 = Keines
 *             1 = Lehner kleines
 *             2 = Lehner grosses
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeDisplay(short deviceID, unsigned char &val, int checked)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetTypeDisplay [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	// Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
	// - Funktionen TypeTransponder liefern <-> Displaywert
	// - Funktionen TypeDisplay liefern <-> Keypadwert
	// - Funktionen TypeKeypad liefern <-> Transponderwert
	//
	// Die Verwechslung wird in der Ableitung dieser Moduleklasse abgefangen.

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DISPLAY, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DISPLAY);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetTypeDisplay [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DISPLAY, &m_hwInfo.typeDisplay, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetTypeDisplay [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Ermittelte Variante.
 *             0 = Keine
 *             1 = PZE - Master IV
 *             2 = BDE - Master IV
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeKeypad(short deviceID, unsigned char &val, int checked)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetTypeKeypad [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	// Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
	// - Funktionen TypeTransponder liefern <-> Displaywert
	// - Funktionen TypeDisplay liefern <-> Keypadwert
	// - Funktionen TypeKeypad liefern <-> Transponderwert
	//
	// Die Verwechslung wird in der Ableitung dieser Moduleklasse abgefangen.

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_KEYPAD, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_KEYPAD);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetTypeKeypad [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_KEYPAD, &m_hwInfo.typeKeypad, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetTypeKeypad [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
 *             0 = Keiner
 *             1 = TSR30
 *             2 = TSR32
 *             3 = PHG PRIMO 100
 *             4 = Cavitec Mifare
 *             5 = Dallas iButton
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeTransponder(short deviceID, unsigned char &val, int checked)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetTypeTransponder [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	// Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
	// - Funktionen TypeTransponder liefern <-> Displaywert
	// - Funktionen TypeDisplay liefern <-> Keypadwert
	// - Funktionen TypeKeypad liefern <-> Transponderwert
	//
	// Die Verwechslung wird in der Ableitung dieser Moduleklasse abgefangen.

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TRANSPONDER, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TRANSPONDER);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetTypeTransponder [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TRANSPONDER, &m_hwInfo.typeTransponder, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetTypeTransponder [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Ermittelte Variante.
 *             0 = Keine
 *             1 = Fest integriert
 *             2 = Herausnehmbar
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeMMC(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetTypeMMC [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MMC, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MMC);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetTypeMMC [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MMC, &m_hwInfo.typeMMC, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetTypeMMC [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Ermittelte Variante.
 *             0 = Keine
 *             1 = Bluegiga (Bluetooth)
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeRadio(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetTypeRadio [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_RADIO, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_RADIO);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetTypeRadio [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_RADIO, &m_hwInfo.typeRadio, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetTypeRadio [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichneten Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Ermittelte Variante.
 *              0 = Kein
 *              1 = Furuno GN-80
 *            ... = ...
 */
int CChannel4ModuleHardware::GetTypeGPS(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetTypeGPS [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_GPS, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_GPS);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetTypeGPS [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_GPS, &m_hwInfo.typeGPS, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetTypeGPS [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetComponentUSB(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetComponentUSB [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_USB, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_USB);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetComponentUSB [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_USB, &m_hwInfo.componentUSB, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetComponentUSB [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceID Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int CChannel4ModuleHardware::GetComponentBattery(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetComponentBattery [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_BATTERY, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_BATTERY);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetComponentBattery [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_BATTERY, &m_hwInfo.componentBattery, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetComponentBattery [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceID Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int CChannel4ModuleHardware::GetComponentCharge(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetComponentCharge [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_CHARGE, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_CHARGE);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetComponentCharge [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_CHARGE, &m_hwInfo.componentCharge, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetComponentCharge [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Ermittelte Variante.
 *             0 = Keine
 *             1 = HKW
 *             4 = Nicht zu verwenden, da auf 0 gemappt.
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeDCF77(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetTypeDCF77 [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DCF77, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DCF77);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetTypeDCF77 [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		// Wert wurde von FWV 04.01.01 fuer die Baudrate von Bluetooth verwendet und
		// steht nach initialisierung auf dem Wert 4. Dieser Wert, wird als Typ ignoriert.
		val = (m_data[0] != 0xFF && m_data[0] != 4) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DCF77, &m_hwInfo.typeDCF77, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetTypeDCF77 [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetComponentInAnalog(short deviceID, unsigned int &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetComponentInAnalog [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_ANALOG, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_ANALOG);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned int))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetComponentInAnalog [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (*(unsigned int*)m_data != 0xFFFFFFFF) ? *(unsigned int*)m_data : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned int);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_ANALOG, (unsigned char*)&m_hwInfo.componentInAnalog, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetComponentInAnalog [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetComponentInDigital(short deviceID, unsigned int &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetComponentInDigital [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_DIGITAL, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_DIGITAL);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned int))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetComponentInDigital [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (*(unsigned int*)m_data != 0xFFFFFFFF) ? *(unsigned int*)m_data : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned int);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_DIGITAL, (unsigned char*)&m_hwInfo.componentInDigital, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetComponentInDigital [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetComponentOutDigital(short deviceID, unsigned int &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetComponentOutDigital [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned int))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetComponentOutDigital [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (*(unsigned int*)m_data != 0xFFFFFFFF) ? *(unsigned int*)m_data : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned int);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL, (unsigned char*)&m_hwInfo.componentOutDigital, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetComponentOutDigital [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Option enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetOptionIOModule(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetOptionIOModule [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_IOMODULE, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_IOMODULE);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetOptionIOModule [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = m_data[0];
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_IOMODULE, &m_hwInfo.optionIOModule, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetOptionIOModule [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Option enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetOptionComServer(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetOptionComServer [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_COMSERVER, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_COMSERVER);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetOptionComServer [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = m_data[0];
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_COMSERVER, &m_hwInfo.optionComserver, count)) != IV_ERROR_SUCCESS)
		{	
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetOptionComServer [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}

		return 0x0200;
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Option enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetOptionTBAdaption(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetOptionTBAdaption [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TBADAPTION, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TBADAPTION);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetOptionTBAdaption [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = m_data[0];
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TBADAPTION, &m_hwInfo.optionTBAdaption, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetOptionTBAdaption [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceID Anzusprechende Geraetenummer.
*/
int CChannel4ModuleHardware::CommandInspectionStation(short deviceID, unsigned char address, char *data, int sizeData, unsigned char &addressAnswer, char *dataAnswer, int &sizeAnswer, int toPending)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::CommandInspectionStation [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceID, address, data, sizeData);

	int count = 0, i;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Parameter eintragen
		// Erster Parameter ist die Adresse
		m_data[0] = address;
		count = sizeof(unsigned char);
		// Folgender Parameter ist eine Optionale Zeichenkette
		for (i=0; i < sizeData; i++)
		{
			m_data[count + i] = data[i];
		}
		count += sizeData;
		// Nullterminierung sicherstellen
		if (m_data[count] != '\0')
		{
			count++;
			m_data[count] = '\0';
		}

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION);
			return m_result;
		}

		// OK
		if (count)
		{
			Trace("%s(%i):WARNING(3): CChannel4ModuleHardware::CommandInspectionStation [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, count);
		}

		// Aktive Antwort abwarten
		if ((m_result = Execute(IV_CONTEXT_SLAVE, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION_OK, m_data, count, ((toPending*1000) / GetTimeout()))) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION);
			return m_result;
		}

		// Erstes Byte ist die Adresse
		addressAnswer = m_data[0];
		// Folgend ein optionaler String
		for (i=0; i < (count-1) && i < sizeAnswer; i++)
		{
			dataAnswer[i] = m_data[1 + i];
		}
		// Laenge der Zeichenkette
		sizeAnswer = i;
	}
	else 
	{
		unsigned char tmp[4];
		count = 0;

		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION, tmp, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::CommandInspectionStation [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
	}

	return m_result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceID Anzusprechende Geraetenummer.
*/
int CChannel4ModuleHardware::CommandInspectionStationOK(short deviceID)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::CommandInspectionStationOK [ %i ]\n", __FILE__, __LINE__, deviceID);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::CommandInspectionStationOK\n", __FILE__, __LINE__);
	}
	else
	{
		unsigned char tmp[4];
		count = 0;

		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION_OK, tmp, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::CommandInspectionStationOK [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
	}

	return m_result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceID Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int CChannel4ModuleHardware::GetOptionTPWrite(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetOptionTPWrite [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TPWRITE, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TPWRITE);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetOptionTPWrite [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = m_data[0];
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TPWRITE, &m_hwInfo.optionTPWrite, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetOptionTPWrite [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceID Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int CChannel4ModuleHardware::GetOptionHTTP(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetOptionHTTP [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_HTTP, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_HTTP);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetOptionHTTP [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = m_data[0];
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_HTTP, &m_hwInfo.optionHttp, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetOptionHTTP [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceID Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int CChannel4ModuleHardware::GetOptionLanguage(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetOptionLanguage [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_LANGUAGE, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_LANGUAGE);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetOptionLanguage [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = m_data[0];
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_LANGUAGE, &m_hwInfo.optionLanguage, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetOptionLanguage [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceID Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int CChannel4ModuleHardware::GetOptionFingerprintUpdate(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetOptionFingerprintUpdate [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_BIOKEY_UPDATE, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_BIOKEY_UPDATE);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetOptionFingerprintUpdate [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = m_data[0];
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_BIOKEY_UPDATE, &m_hwInfo.optionBiokeyUpdate, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetOptionFingerprintUpdate [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Ermittelte Variante.
 *             0 = Keine
 *             1 = Idencom (BioKey 3000)
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeFingerprint(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetTypeFingerprint [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_FINGERPRINT, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_FINGERPRINT);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetTypeFingerprint [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_FINGERPRINT, &m_hwInfo.typeFingerprint, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetTypeFingerprint [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Bitfeld der verwendbaren Varianten.
 *            -1 = Undefiniert
 *            Bit 1 = GIS
 *            Bit 2 = Protect
 *            Bit 3 = PHG
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeExternalReader(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetTypeExternalReader [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_EXTERNAL_READER, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_EXTERNAL_READER);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetTypeExternalReader [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// BUGFIX
		// Eine Firmware die die Serien TS und Protect untersttzt setzt als untersttzte Serien
		// TS und Protect obwohl es so sein sollte, dass die angesprochene Serie eingetragen wird,
		// so dass der Wert der Hardwareinfo eindeutig ist.

		// Wert uebernehmen
		val = m_data[0];

		// Werden beide Flags vorgefunden wird von TS alleine ausgegangen.
		if ((val != 0xFF) && (val & IV_VALUE_TYPE_EXTERNAL_READER_GIS) && (val & IV_VALUE_TYPE_EXTERNAL_READER_PROTECT))
		{
			val = IV_VALUE_TYPE_EXTERNAL_READER_GIS;
		}
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_EXTERNAL_READER, &m_hwInfo.typeExternalReader, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetTypeExternalReader [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichneten Komponente enthalten ist.
*
* @param deviceID Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*              0 = Kein
*              1 = Exklusiv-Line IV
*              2 = Flex-MasterIV
*            ... = ...
*/
int CChannel4ModuleHardware::GetTypeTouch(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetTypeTouch [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TOUCH, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TOUCH);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetTypeTouch [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TOUCH, &m_hwInfo.typeTouch, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetTypeTouch [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichneten Komponente enthalten ist.
*
* @param deviceID Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*              0 = Kein
*              1 = Vibration
*              2 = Beschleunigung
*            ... = ...
*/
int CChannel4ModuleHardware::GetTypeSensor(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetTypeSensor [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_SENSOR, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_SENSOR);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetTypeSensor [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_SENSOR, &m_hwInfo.typeSensor, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetTypeSensor [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
* @brief Setzen der Optionen fuer Setup.
*
* @param deviceID Anzusprechende Geraetenummer.
* @param mask Gibt die gltigen Werte des Wertes val an.
* @param val Gibt an wie die Option gesetzt werden soll.
*         bit 0 = Signalverarbeitung
*           ... = ...
*/
int CChannel4ModuleHardware::SetSetupOptions(short deviceID, unsigned short mask, unsigned short options)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::SetSetupOptions [ %i, 0x%04X, 0x%04X ]\n", __FILE__, __LINE__, deviceID, mask, options);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Die Komponendenangabe uebernehmen.
		*(unsigned short*)&m_data[0] = mask;
		count += sizeof(unsigned short);

		*(unsigned short*)&m_data[2] = options;
		count += sizeof(unsigned short);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count)
		{
			Trace("WARNING(3): CChannel4ModuleHardware::SetSetupOptions [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", count);
		}
	}
	else
	{
		int result;
		// Parametergroesse setzen
		count = 0;

		unsigned char val = 0;
		// Wert an interne Struktur uebernehmen
		m_hwInfo.optionsSetup = options;	

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS, &val, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::SetSetupOptions [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
* @brief Ermitteln der Optionen fuer Setup.
*
* @param deviceID Anzusprechende Geraetenummer.
* @param mask Gibt die gltigen Werte des Wertes val an.
* @param val Gibt an wie die Option gesetzt werden soll.
*         bit 0 = Signalverarbeitung
*           ... = ...
*/
int CChannel4ModuleHardware::GetSetupOptions(short deviceID, unsigned short &mask, unsigned short &options)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetSetupOptions [ %i, 0x%04X, 0x%04X ]\n", __FILE__, __LINE__, deviceID, mask, options);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_SETUP_OPTIONS, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_GET_SETUP_OPTIONS);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count != (2 * sizeof(unsigned short)))
		{
			Trace("WARNING(3): CChannel4ModuleHardware::GetSetupOptions [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", count);
		}

		// Die Komponendenangabe uebernehmen.
		mask = *(unsigned short*)&m_data[0];
		options = *(unsigned short*)&m_data[2];
	}
	else 
	{
		int result;
		// Parametergroesse setzen
		count = 0;

		unsigned char val;
		// Wert an interne Struktur uebernehmen
		options = m_hwInfo.optionsSetup;	

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS, &val, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::SetSetupOptions [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt die "Version" der Hardwareconfiguration.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 * @param val Wert fuer Version der Hardwareconfiguration.
 */
int CChannel4ModuleHardware::GetHWConfig(short deviceID, unsigned char &val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::GetHWConfig [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HW_CONFIG, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HW_CONFIG);
			return m_result;
		}

		// Antwortparameter pruefen
		if (count != sizeof(unsigned char))
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleHardware::GetHWConfig [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
			val = 0;
			return m_result;
		}

		// Wert uebernehmen
		val = (m_data[0] != 0xFF) ? m_data[0] : 0;
	}
	else 
	{
		int result;
		// Parametergroesse setzen
		count = sizeof(unsigned char);

		unsigned char tmp[1];
		tmp[0] = 1;
		
		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HW_CONFIG, tmp, count)) != IV_ERROR_SUCCESS)
		{			
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::GetHWConfig [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}


#ifndef DATAFOX_STUDIO_IV

// Funktions-Stumps

int CChannel4ModuleHardware::PingRandom(short deviceID)                                 { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }

int CChannel4ModuleHardware::SetTypeDevice(short deviceID, unsigned char val, unsigned short val2)         { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetTypeBoard(short deviceID, unsigned char val, unsigned short val2)          { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetTypeTCPIP(short deviceID, unsigned char val, unsigned short val2)          { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetTypeModemMC(short deviceID, unsigned char val, unsigned short val2)        { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetTypeDisplay(short deviceID, unsigned char val, unsigned short val2, int checked)     { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetTypeKeypad(short deviceID, unsigned char val, unsigned short val2, int checked)      { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetTypeTransponder(short deviceID, unsigned char val, unsigned short val2, int checked) { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetTypeMMC(short deviceID, unsigned char val, unsigned short val2)            { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetTypeRadio(short deviceID, unsigned char val, unsigned short val2)          { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetTypeGPS(short deviceID, unsigned char val, unsigned short val2)            { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetTypeDCF77(short deviceID, unsigned char val, unsigned short val2)          { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetTypeFingerprint(short deviceID, unsigned char val, unsigned short val2)    { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetTypeExternalReader(short deviceID, unsigned char val, unsigned short val2) 
{
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::SetTypeExternalReader [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;
	unsigned short key;
	switch (val) 
	{
		case 0xFF: key = 0xCF38; break;
		case 0x00: key = 0x8F78; break;
		case 0x01: key = 0x4FB9; break;
		case 0x02: key = 0x4EF9; break;
		default: return -IV_ERROR_FUNCTION_PARAMETER;
	}
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Die Komponendenangabe uebernehmen.
		m_data[0] = val;
		count += sizeof(unsigned char);

		*(unsigned short*)&m_data[1] = key;
		count += sizeof(unsigned short);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_TYPE_EXTERNAL_READER, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_TYPE_EXTERNAL_READER);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count)
		{
			Trace("WARNING(3): CChannel4ModuleHardware::SetTypeExternalReader [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", count);
		}
	}
	else 
	{
		// Schluessel pruefen: Funktionsfehler
		if (key != val2) return 0x0600;

		int result;
		// Parametergroesse setzen
		count = 0;

		// Wert an interne Struktur uebernehmen
		m_hwInfo.optionBiokeyUpdate = val;	

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_TYPE_EXTERNAL_READER, &val, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::SetTypeExternalReader [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}
int CChannel4ModuleHardware::SetTypeTouch(short deviceID, unsigned char val, unsigned short val2)          { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetTypeSensor(short deviceID, unsigned char val, unsigned short val2)         { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }

int CChannel4ModuleHardware::SetComponentRS485_0(short deviceID, unsigned char val, unsigned short val2)   { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetComponentRS485_1(short deviceID, unsigned char val, unsigned short val2)   { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetComponentUSB(short deviceID, unsigned char val, unsigned short val2)       { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetComponentInAnalog(short deviceID, unsigned int val, unsigned short val2)  { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetComponentInDigital(short deviceID, unsigned int val, unsigned short val2) { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetComponentOutDigital(short deviceID, unsigned int val, unsigned short val2){ if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetComponentBattery(short deviceID, unsigned char val, unsigned short val2)   { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }
int CChannel4ModuleHardware::SetComponentCharge(short deviceID, unsigned char val, unsigned short val2)    { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }

int CChannel4ModuleHardware::SetOptionIOModule(short deviceID, unsigned char val, unsigned short val2)     
{ 
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::SetOptionIOModule [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;
	unsigned short key;
	switch (val)
	{
		case 0xFF: key = 0x0C59; break;
		case 0x00: key = 0x4C19; break;
		case 0x01: key = 0x8CD8; break;
		default: return -IV_ERROR_FUNCTION_PARAMETER;
	}
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Die Komponendenangabe uebernehmen.
		m_data[0] = val;
		count += sizeof(unsigned char);

		*(unsigned short*)&m_data[1] = key;
		count += sizeof(unsigned short);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_IOMODULE, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_IOMODULE);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count)
		{
			Trace("WARNING(3): CChannel4ModuleHardware::SetOptionIOModule [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", count);
		}
	}
	else 
	{
		// Schluessel pruefen: Funktionsfehler
		if (key != val2) return 0x0600;

		int result;
		// Parametergroesse setzen
		count = 0;

		// Wert an interne Struktur uebernehmen
		m_hwInfo.optionIOModule = val;	

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_IOMODULE, &val, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::SetOptionIOModule [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

int CChannel4ModuleHardware::SetOptionComServer(short deviceID, unsigned char val, unsigned short val2)    
{ 
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::SetOptionComServer [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;
	unsigned short key;
	switch (val)
	{
		case 0xFF: key = 0xCCF8; break;
		case 0x00: key = 0x8CB8; break;
		case 0x01: key = 0x4C79; break;
		default: return -IV_ERROR_FUNCTION_PARAMETER;
	}
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Die Komponendenangabe uebernehmen.
		m_data[0] = val;
		count += sizeof(unsigned char);

		*(unsigned short*)&m_data[1] = key;
		count += sizeof(unsigned short);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_COMSERVER, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_COMSERVER);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count)
		{
			Trace("WARNING(3): CChannel4ModuleHardware::SetOptionComServer [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", count);
		}
	}
	else
	{
		// Schluessel pruefen: Funktionsfehler
		if (key != val2) return 0x0600;

		int result;
		// Parametergroesse setzen
		count = 0;

		// Wert an interne Struktur uebernehmen
		m_hwInfo.optionComserver = val;	

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_COMSERVER, &val, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::SetOptionComServer [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

int CChannel4ModuleHardware::SetOptionTBAdaption(short deviceID, unsigned char val, unsigned short val2)   
{ 
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::SetOptionTBAdaption [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;
	unsigned short key;
	switch (val)
	{
		case 0xFF: key = 0xCD18; break;
		case 0x00: key = 0x8D58; break;
		case 0x01: key = 0x4D99; break;
		default: return -IV_ERROR_FUNCTION_PARAMETER;
	}
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Die Komponendenangabe uebernehmen.
		m_data[0] = val;
		count += sizeof(unsigned char);

		*(unsigned short*)&m_data[1] = key;
		count += sizeof(unsigned short);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TBADAPTION, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TBADAPTION);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count)
		{
			Trace("WARNING(3): CChannel4ModuleHardware::SetOptionTBAdaption [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", count);
		}
	}
	else 
	{
		// Schluessel pruefen: Funktionsfehler
		if (key != val2) return 0x0600;

		int result;
		// Parametergroesse setzen
		count = 0;

		// Wert an interne Struktur uebernehmen
		m_hwInfo.optionTBAdaption = val;	

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TBADAPTION, &val, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::SetOptionTBAdaption [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

int CChannel4ModuleHardware::SetOptionTPWrite(short deviceID, unsigned char val, unsigned short val2)   
{ 
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::SetOptionTPWrite [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;
	unsigned short key;
	switch (val)
	{
		case 0xFF: key = 0x0BD9; break;
		case 0x00: key = 0x4B99; break;
		case 0x01: key = 0x8B58; break;
		default: return -IV_ERROR_FUNCTION_PARAMETER;
	}
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Die Komponendenangabe uebernehmen.
		m_data[0] = val;
		count += sizeof(unsigned char);

		*(unsigned short*)&m_data[1] = key;
		count += sizeof(unsigned short);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TPWRITE, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TPWRITE);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count)
		{
			Trace("WARNING(3): CChannel4ModuleHardware::SetOptionTPWrite [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", count);
		}
	}
	else 
	{
		// Schluessel pruefen: Funktionsfehler
		if (key != val2) return 0x0600;

		int result;
		// Parametergroesse setzen
		count = 0;

		// Wert an interne Struktur uebernehmen
		m_hwInfo.optionTPWrite = val;	

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TPWRITE, &val, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::SetOptionTPWrite [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

int CChannel4ModuleHardware::SetOptionHTTP(short deviceID, unsigned char val, unsigned short val2)   
{ 
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::SetOptionHTTP [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);

	int count = 0;
	unsigned short key;
	switch (val)
	{
		case 0xFF: key = 0xCB78; break;
		case 0x00: key = 0x8B38; break;
		case 0x01: key = 0x4BF9; break;
		default: return -IV_ERROR_FUNCTION_PARAMETER;
	}
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Die Komponendenangabe uebernehmen.
		m_data[0] = val;
		count += sizeof(unsigned char);

		*(unsigned short*)&m_data[1] = key;
		count += sizeof(unsigned short);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_HTTP, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_HTTP);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count)
		{
			Trace("WARNING(3): CChannel4ModuleHardware::SetOptionHTTP [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", count);
		}
	}
	else 
	{
		// Schluessel pruefen: Funktionsfehler
		if (key != val2) return 0x0600;

		int result;
		// Parametergroesse setzen
		count = 0;

		// Wert an interne Struktur uebernehmen
		m_hwInfo.optionHttp = val;	

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_HTTP, &val, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::SetOptionHTTP [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}

int CChannel4ModuleHardware::SetOptionLanguage(short deviceID, unsigned char val, unsigned short val2)   { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }

int CChannel4ModuleHardware::SetOptionBiokeyUpdate(short deviceID, unsigned char val, unsigned short val2)   
{ 
	Trace("%s(%i):INFO(3): CChannel4ModuleHardware::SetOptionBiokeyUpdate [ %i, %i ]\n", __FILE__, __LINE__, deviceID, val);
	int count = 0;
	unsigned short key;
	switch (val)
	{
		case 0xFF: key = 0x0A39; break;
		case 0x00: key = 0x4A79; break;
		case 0x01: key = 0x8AB8; break;
		default: return -IV_ERROR_FUNCTION_PARAMETER;
	}
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Die Komponendenangabe uebernehmen.
		m_data[0] = val;
		count += sizeof(unsigned char);

		*(unsigned short*)&m_data[1] = key;
		count += sizeof(unsigned short);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_BIOKEY_UPDATE, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_BIOKEY_UPDATE);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count)
		{
			Trace("WARNING(3): CChannel4ModuleHardware::SetOptionBiokeyUpdate [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", count);
		}
	}
	else 
	{
		// Schluessel pruefen: Funktionsfehler
		if (key != val2) return 0x0600;

		int result;
		// Parametergroesse setzen
		count = 0;

		// Wert an interne Struktur uebernehmen
		m_hwInfo.optionBiokeyUpdate = val;	

		// Antwort des Befehls senden.
		if ((result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_BIOKEY_UPDATE, &val, count)) != IV_ERROR_SUCCESS)
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleHardware::SetOptionBiokeyUpdate [ Execute (%i) ]\n", __FILE__, __LINE__, result);
			return result;
		}
	}

	return m_result;
}


int CChannel4ModuleHardware::SetHWConfig(short deviceID, unsigned char val, unsigned short val2)           { if (GetDeviceID() != deviceID) return -IV_ERROR_FUNCTION_UNAVAILABLE; else return 0x0200; }

#endif

