#include "stdafx.h"
#include "DFSoftwareVersion.h"

// Zeichenfolgen die den Geraetetyp angeben
const char CDFSoftwareVersion::m_typeDeviceText[VF_MAX_DEVICES][VF_MAX_DEVICE_NAME] = {VF_DEVICE_NAME_PZE, VF_DEVICE_NAME_ZK, VF_DEVICE_NAME_BDE, VF_DEVICE_NAME_MOBIL, VF_DEVICE_NAME_MDE, VF_DEVICE_NAME_AE, VF_DEVICE_NAME_TIMEBOY, VF_DEVICE_NAME_EXKLUSIV, VF_DEVICE_NAME_FLEX };

CDFSoftwareVersion::CDFSoftwareVersion()
{
	Reset();
}

CDFSoftwareVersion::CDFSoftwareVersion(const char *version)
{
	Reset();
	if (!Parse(version))
	{
		Reset();
	}
}

CDFSoftwareVersion::CDFSoftwareVersion(unsigned int version)
{
	Reset();
	if (!Parse(version))
	{
		Reset();
	}
}

CDFSoftwareVersion::CDFSoftwareVersion(unsigned char master, char major, unsigned char minor, unsigned char build, unsigned char typeDevice)
{
	char typeDeviceText[VF_MAX_DEVICE_NAME];
	Reset();

	if (master > 99) master = 99;
	m_master = master;
	if (major > 99) major = 99;
	m_major = major;
	if (minor > 99) minor = 99;
	m_minor = minor;
	if (build > 99) build = 99;
	m_build = build;
	if (typeDevice > VF_MAX_DEVICES)
	{
		sprintf(typeDeviceText, "%i", typeDevice);
	}
	else
	{
		strcpy(typeDeviceText, m_typeDeviceText[typeDevice]);
	}
	m_typeDevice = typeDevice;
	sprintf(m_versionFirmware, "%02i.%02i.%02i.%02i.%s", m_master, m_major, m_minor, m_build, typeDeviceText);
}

CDFSoftwareVersion::~CDFSoftwareVersion()
{
}


int CDFSoftwareVersion::Parse(const char *version)
{
	// Formatvorlage kurz: XX.XX.XX???
	// Formatvorlage lang: XX.XX.XX.XX.YYYYYYY
	if (strlen(version) < 8 || strlen(version) > sizeof(m_versionFirmware) ||
		!isdigit(version[0]) || !isdigit(version[1]) || 
		version[2] != '.' || !isdigit(version[3]) || !isdigit(version[4]) || 
		version[5] != '.' || !isdigit(version[6]) || !isdigit(version[7]) || 
		(strlen(version) > 8 && (version[8] != '.' || !isdigit(version[9]) || !isdigit(version[10]))) ) {

		// Formatfehler
		return 0;
	}

	// Kopie des Versionsstrings halten.
	memset(m_versionFirmware, 0, sizeof(m_versionFirmware));
	strcpy(m_versionFirmware, version);

	int i;
	// Alle Werte bernehmen
	i = 0;
	m_master = (version[i] - _T('0')) * 10 + (version[i + 1] - _T('0'));
	i += 3;
	m_major  = (version[i] - _T('0')) * 10 + (version[i + 1] - _T('0'));
	i += 3;
	m_minor  = (version[i] - _T('0')) * 10 + (version[i + 1] - _T('0'));
	m_build = 0;
	if (strlen(version) > 8)
	{	
		i += 3;
		m_build  = (version[i] - _T('0')) * 10 + (version[i + 1] - _T('0'));
	}

	m_typeDevice = VF_DEVICE_NA;
	// Ermittlung des Geraetetyps?
	if (strlen(version) > 12)
	{
		for (i=0; i < VF_MAX_DEVICES; i++)
		{
			if (strcmp(&version[12], m_typeDeviceText[i]) == 0)
			{
				m_typeDevice = i;
				break;
			}
		}
	}

	return 1;
}

int CDFSoftwareVersion::Parse(unsigned int version)
{
	if ((((version >> 24) & 0xFF) > 99) ||
		(((version >> 16) & 0xFF) > 99) ||
		(((version >>  8) & 0xFF) > 99) ||
		((version & 0xFF) > 99) )
	{
		// Formatfehler
		return 0;
	}

	// Wert in Member uebernehmen
	m_master = (version >> 24) & 0xFF;
	m_major  = (version >> 16) & 0xFF;
	m_minor  = (version >>  8) & 0xFF;
	m_build  = version & 0xFF;
	m_typeDevice = VF_DEVICE_NA;

	memset(m_versionFirmware, 0, sizeof(m_versionFirmware));
	sprintf(m_versionFirmware, "%02i.%02i.%02i.%02i", m_master, m_major, m_minor, m_build);

	return 1;
}

