#ifndef _DFSOFTWAREVERSION_
#define _DFSOFTWAREVERSION_

#ifdef _WIN32
#pragma warning(disable : 4996)
#else
#define _T(x) x
#endif

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <memory.h>

// Maximale Anzahl
#define VF_MAX_DEVICES                 9
#define VF_MAX_DEVICE_NAME            10

#define VF_DEVICE_NA                   0xFF
#define VF_DEVICE_PZE                  0
#define VF_DEVICE_ZK                   1
#define VF_DEVICE_BDE                  2
#define VF_DEVICE_MOBIL                3
#define VF_DEVICE_MDE                  4
#define VF_DEVICE_AE                   5
#define VF_DEVICE_TIMEBOY              6
#define VF_DEVICE_EXKLUSIV             7
#define VF_DEVICE_FLEX                 8

#define IV_DEVICE_NAME_NA              "NA"
#define VF_DEVICE_NAME_PZE             "PZE"
#define VF_DEVICE_NAME_ZK              "ZK"
#define VF_DEVICE_NAME_BDE             "BDE"
#define VF_DEVICE_NAME_MOBIL           "MOBIL"
#define VF_DEVICE_NAME_MDE             "MDE"
#define VF_DEVICE_NAME_AE              "AE"
#define VF_DEVICE_NAME_TIMEBOY         "TIMEBOY"
#define VF_DEVICE_NAME_EXKLUSIV        "EXKLUSIV"
#define VF_DEVICE_NAME_FLEX            "FLEX"

class CDFSoftwareVersion
{
public:
	CDFSoftwareVersion();
	CDFSoftwareVersion(const char *version);
	CDFSoftwareVersion(unsigned int version);
	CDFSoftwareVersion(unsigned char master, char major, unsigned char minor, unsigned char build, unsigned char typeDevice);
	virtual ~CDFSoftwareVersion();
  friend class CDFSoftwarebuildVersion;

private:
	static const char m_typeDeviceText[VF_MAX_DEVICES][VF_MAX_DEVICE_NAME];
	char m_versionFirmware[64];
	unsigned char m_master;
	unsigned char m_major;
	unsigned char m_minor;
	unsigned char m_build;
	unsigned char m_typeDevice;
	
public:
	void Reset()
	{
		m_master = m_major = m_minor = m_build = 0;
		m_typeDevice = 0;
		memset(m_versionFirmware, 0, sizeof(m_versionFirmware));
		sprintf(m_versionFirmware, "%02i.%02i.%02i.%02i", m_master, m_major, m_minor, m_build);
	}
	int Parse(const char *version);
	int Parse(unsigned int version);

  inline unsigned char GetMaster() {
    return m_master;
  }
  inline unsigned char GetMajor() {
    return m_major;
  }
  inline unsigned char GetMinor() {
    return m_minor;
  }
  inline unsigned char GetBuild() {
    return m_build;
  }
	inline unsigned char GetTypeDevice() {
		return m_typeDevice;
	}
public: // Zuweisungsoperator
	CDFSoftwareVersion& operator= (const char *version)
	{
		if (!this->Parse(version))
		{
			Reset();
		}
		return *this;
	}
	CDFSoftwareVersion& operator= (unsigned int version)
	{
		if (!this->Parse(version))
		{
			Reset();
		}
		return *this;
	}


public: // Vergleichsoperatoren
	// Gleichheit
	bool operator== (CDFSoftwareVersion &version)
	{
		if (this->m_master == version.m_master &&
			this->m_major == version.m_major &&
			this->m_minor == version.m_minor)
		{
			return true;
		}

		return false;
	}
	bool operator== (const char *version)
	{
		CDFSoftwareVersion lVersion(version);
		return (*this == lVersion);
	}
	// Ungleichheit
	bool operator!= (CDFSoftwareVersion &version)
	{
		if (this->m_master != version.m_master ||
			this->m_major != version.m_major ||
			this->m_minor != version.m_minor)
		{
			return true;
		}

		return false;
	}
	bool operator!= (const char *version)
	{
		CDFSoftwareVersion lVersion(version);
		return (*this != lVersion);
	}
	// Groesser
	bool operator> (CDFSoftwareVersion &version)
	{
		if (this->m_master > version.m_master ||
			(this->m_master == version.m_master && this->m_major > version.m_major) ||
			(this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor > version.m_minor))
		{
			return true;
		}

		return false;
	}
	bool operator> (const char *version)
	{
		CDFSoftwareVersion lVersion(version);
		return (*this > lVersion);
	}
	// Groesser gleich
	bool operator>= (CDFSoftwareVersion &version)
	{
		if (*this > version || *this == version)
		{
			return true;
		}

		return false;
	}
	bool operator>= (const char *version)
	{
		CDFSoftwareVersion lVersion(version);
		return (*this >= lVersion);
	}
	// Kleiner
	bool operator< (CDFSoftwareVersion &version)
	{
		if (this->m_master < version.m_master ||
			(this->m_master == version.m_master && this->m_major < version.m_major) ||
			(this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor < version.m_minor))
		{
			return true;
		}

		return false;
	}
	bool operator< (const char *version)
	{
		CDFSoftwareVersion lVersion(version);
		return (*this < lVersion);
	}
	// Kleiner gleich
	bool operator<= (CDFSoftwareVersion &version)
	{
		if (*this < version || *this == version)
		{
			return true;
		}

		return false;
	}
	bool operator<= (const char *version)
	{
		CDFSoftwareVersion lVersion(version);
		return (*this <= lVersion);
	}

public: // Cast operatoren
	operator char* const()
	{
		return m_versionFirmware;
	}
};

class CDFSoftwarebuildVersion : public CDFSoftwareVersion
{
public:
  CDFSoftwarebuildVersion() : CDFSoftwareVersion() {

  }
  CDFSoftwarebuildVersion(const char *version) : CDFSoftwareVersion(version) {

  }
  CDFSoftwarebuildVersion(unsigned int version) : CDFSoftwareVersion(version) {

  }
  CDFSoftwarebuildVersion(unsigned char master, char major, unsigned char minor, unsigned char build, unsigned char typeDevice) : CDFSoftwareVersion(master, major, minor, build, typeDevice) {
  }

public: // Vergleichsoperatoren
  // Gleichheit
  bool operator== (CDFSoftwarebuildVersion &version) {
    if (this->m_master == version.m_master &&
      this->m_major == version.m_major &&
      this->m_minor == version.m_minor &&
      this->m_build == version.m_build) {
      return true;
    }

    return false;
  }
  bool operator== (const char *version) {
    CDFSoftwarebuildVersion lVersion(version);
    return (*this == lVersion);
  }
  // Ungleichheit
  bool operator!= (CDFSoftwarebuildVersion &version) {
    if (this->m_master != version.m_master ||
      this->m_major != version.m_major ||
      this->m_minor != version.m_minor ||
      this->m_build != version.m_build) {
      return true;
    }

    return false;
  }
  bool operator!= (const char *version) {
    CDFSoftwarebuildVersion lVersion(version);
    return (*this != lVersion);
  }
  // Groesser
  bool operator> (CDFSoftwarebuildVersion &version) {
    if (this->m_master > version.m_master ||
      (this->m_master == version.m_master && this->m_major > version.m_major) ||
      (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor > version.m_minor) ||
      (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor == version.m_minor && this->m_build > version.m_build)) {
      return true;
    }

    return false;
  }
  bool operator> (const char *version) {
    CDFSoftwarebuildVersion lVersion(version);
    return (*this > lVersion);
  }
  // Groesser gleich
  bool operator>= (CDFSoftwarebuildVersion &version)
  {
    if (*this > version || *this == version)
    {
      return true;
    }

    return false;
  }
  bool operator>= (const char *version)
  {
    CDFSoftwarebuildVersion lVersion(version);
    return (*this >= lVersion);
  }
  // Kleiner
  bool operator< (CDFSoftwarebuildVersion &version) {
    if (this->m_master < version.m_master ||
      (this->m_master == version.m_master && this->m_major < version.m_major) ||
      (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor < version.m_minor) ||
      (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor == version.m_minor && this->m_build < version.m_build)) {
      return true;
    }

    return false;
  }
  bool operator< (const char *version) {
    CDFSoftwarebuildVersion lVersion(version);
    return (*this < lVersion);
  }
  // Kleiner gleich
  bool operator<= (CDFSoftwarebuildVersion &version)
  {
    if (*this < version || *this == version)
    {
      return true;
    }

    return false;
  }
  bool operator<= (const char *version)
  {
    CDFSoftwarebuildVersion lVersion(version);
    return (*this <= lVersion);
  }

};
#endif
