#ifndef _EXTENDED_FLASH_H_
#define _EXTENDED_FLASH_H_

// ########################################################
// Erweiterter Flash

///////////////////////////////////////
// Struct: StructExtendedFlash

#define EXTENDED_CHECK 0xAAAAAAAE  ///< StructAE3Extended.ulCheck: Pruefwert fuer Gueltigkeit (Versionskennung)

#define EXTF_EMPTY     0x00000000  ///< StructAE3Extended.ulFlags: Nicht vorhanden.
#define EXTF_NEW       0x00000001  ///< StructAE3Extended.ulFlags: Struktur angelegt.
#define EXTF_DOWNLOAD  0x00000010  ///< StructAE3Extended.ulFlags: Aus Geraet ausgelesene Daten.

/** Diese Struktur enthaelt die Kopfdaten des erweiterten Speichers.

  Diese Struktur enthaelt die Adressen der einzelnen Bloecke. Sie ist die erste und einzige Struktur ihrer art
  im Erweiterten Speicherblock (16kByte).
*/
typedef struct StructExtendedFlash {
	unsigned int ulCheck;             ///< muss immer mit EXTENDED_CHECK belegt werden.
	unsigned int ulSize;              ///< groesse der Stuktur sizeof(TExtendedFlash)
	unsigned int pulIOModule;         ///< Zeiger auf IOModul - Struktur
	unsigned int pulIODigital;        ///< Zeiger auf die 8 TDigitalInput - Strukturen des IOModul
	unsigned int pulMDEDigital;       ///< Zeiger auf die 4 TDigitalInput - Strukturen fuer intern.
	unsigned int pulEKArray;          ///< Zeiger auf PointerArray der Eingabeketten fuer MDE
	unsigned int pulFreeScope;        ///< Zeiger auf naechsten freien Speicherbereich im ExtendedFlashMem
	unsigned int pulEntranceTables;   ///< Tabellen fuer Zutrittskontrolle
	unsigned int pulEKArrayEntrance;  ///< Zeiger auf PointerArray der Eingabeketten fuer Zutrittskontrolle
	unsigned int pulValueGVx;         ///< Zeiger auf Struktur mit Standardwerten der GVs
	unsigned int pulEntrance2Tables;  ///< Tabellen fuer Zutrittskontrolle
	unsigned int pulEKArrayEntrance2; ///< Zeiger auf PointerArray der Eingabeketten fuer Zutrittskontrolle
	unsigned int pulExtended;         ///< Zeiger auf Struktur fuer Erweiterte Funktionalitaeten
	unsigned int pulMDEDigitalExt;    ///< Zeiger auf die weiteren 4 TDigitalInput - Strukturen fuer intern.
	unsigned int pulMDEAnalog;        ///< Zeiger auf die 2 TAnalogInput - Strukturen fuer intern.
	unsigned int pulEnergieSchemas;   ///< Zeiger auf die Energieschemen
	unsigned int pulMachineDefs;      ///< Zeiger auf die Maschienendefinitionen
	unsigned int pulEnergieMemo;      ///< Zeiger auf Zeichenfolge fr Memo der Energieverwaltung
	unsigned char tmp[24];             ///< Reserviert. Auf 100 Byte Strukturgroesse bringen.
	unsigned int ulFlags;             ///< Flags fuer Struktur und Erweiterten Speicher
} TExtendedFlash;

///////////////////////////////////////
// Struct: StructHardwareInfo

/** Diese Struktur enthaelt die Informationen zum Geraet.

  Diese Struktur ist z. Z. nur Definiert.
*/
typedef struct StructHardwareInfo {
	unsigned char version;
	unsigned char typeDevice;
	unsigned char typeBoard;
	unsigned char componentRS485_0;
	unsigned char typeTCPIP;
	unsigned char typeModemMC;
	unsigned char componentRS485_1;
	unsigned char typeTransponder;
	unsigned char typeDisplay;
	unsigned char typeKeypad;
	unsigned char typeMMC;
	unsigned char typeGPS;
	unsigned char componentUSB;
	unsigned char typeRadio;
	unsigned char typeDCF77;
	unsigned int componentInAnalog;
	unsigned int componentInDigital;
	unsigned int componentOutDigital;
	unsigned char typeFingerprint;
	unsigned char typeExternalReader;
	unsigned char optionIOModule;
	unsigned char optionTBAdaption;
// Zutrittskontrolle 
#define HIAF_HAS_ZK       0x01 // Geraet nutzt Zutrittskontrolle

#define HIAF_HAS_OPTIONS2 0x02
	unsigned char administrationFlags;
	unsigned char optionComserver;
	unsigned char optionTPWrite;
	unsigned char optionHttp;
	unsigned char optionLanguage;
	unsigned char optionBiokeyUpdate;
#define HIAF_HAS_OPTIONS3 0x04
	unsigned short optionsSetup;
#define HIAF_HAS_OPTIONS4  0x08
	unsigned char typeTouch;
	unsigned char typeSensor;
	unsigned char componentBattery;
	unsigned char componentCharge;
	unsigned char tmp[57];
} TDFHardwareInfo;

///////////////////////////////////////
// Struct: StructAE3IOModule

#define MAX_UNIT_TXT	5 ///< StructAE3IOModule.szUnitU: Maximallaenge des Einheitssuffix mit 0 terminie.

/** Diese Struktur enthaelt die Informationen zum IO-Module.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructIOModule {

	// Spannungseingaenge
	unsigned char ucActiveU;       ///< Flags welcher Eingang aktiv.
	char szNameU[8][MAX_NAME];     ///< Bezeichner der Eingaenge.
	float fMinU[8];                ///< Minimalwert fuer Berechnung.
	float fMaxU[8];                ///< Maximalwert fuer Berechnung.
	char szUnitU[8][MAX_UNIT_TXT]; ///< Einheit - suffix der Berechnung.
	unsigned char ucOnScreenU;     ///< Flags ob Ausgabe auf Display erlaubt.

	// Stromeingaenge
	unsigned char ucActiveI;       ///< Flags welcher Eingang aktiv.
	char szNameI[8][MAX_NAME];     ///< Bezeichner der Eingaenge
	float fMinI[8];                ///< Minimalwert fuer Berechnung.
	float fMaxI[8];                ///< Maximalwert fuer Berechnung.
	char szUnitI[8][MAX_UNIT_TXT]; ///< Einheit - suffix der Berechnung.
	unsigned char ucOnScreenI;     ///< Flags ob Ausgabe auf Display erlaubt.

} TIOModule;


///////////////////////////////////////
// Struct: StructDigitalInput

/** Diese Struktur enthaelt die Informationen fuer die Behandlung eines digitalen Einganges.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructDigitalInput {

	unsigned char ucUseAs;         ///< Verwendung als Start/Stop, Zaehler etc..
	unsigned char ucStartLowHigh;  ///< Auszufuehrende Eingabekette bei Flankenwechsel low -> high
	unsigned char ucStartHighLow;  ///< Auszufuehrende Eingabekette bei Flankenwechsel high -> low
	unsigned short uiCountDivisor; ///< Bei Verwendung als Zaehler, dessen Teiler
	unsigned short uiCycle;        ///< Bei Verwendung als Zaehler, dessen Zyklus.
	unsigned short uiTimeOut;      ///< Bei Verwendung als Zaehler mit Start/Stop, dessen Timeout.
	unsigned int ulCountDivisor32;///< Bei Verwendung als Zaehler, dessen Teiler als 32bit
	unsigned short uiDebounceHigh; ///< Endprellzeit fuer Flankenwechsel Low-High
	unsigned short uiDebounceLow;  ///< Endprellzeit fuer Flankenwechsel High-Low
	unsigned char ucInvokeInputChainCountDivisor; ///< Auszufuehrende Eingabekette bei erreichtem Zaehler
	unsigned char ucInvokeInputChainCycle;        ///< Auszufuehrende Eingabekette bei erreichtem Zyklus

	unsigned char ucTmp[2];       ///< Reserviert.

} TDigitalInput;

///////////////////////////////////////
// Struct: StructDigitalInputExt

/** Diese Struktur enthaelt zusaetzliche Informationen fuer die Behandlung eines digitalen Einganges.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructDigitalInputExt {

	unsigned short uiCavity;   ///< Die zu verwendende Kavitaet.
	unsigned short uiStrokes;  ///< Die zu verwendende Hubzahl.

	unsigned char ucTmp[17];       ///< Reserviert.

} TDigitalInputExt;

///////////////////////////////////////
// Struct: StructAnalogInput

/** Diese Struktur enthaelt die Informationen fuer die Behandlung eines analogen Einganges.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
/*
typedef struct StructAnalogInput {

	unsigned char useAs;            ///< Eingangsverwendung als ...
	char name[MAX_NAME];            ///< Bezeichner der Eingaenge.
	float min;                      ///< Minimalwert fuer Berechnung.
	float max;                      ///< Maximalwert fuer Berechnung.
	char unit[MAX_UNIT_TXT];        ///< Einheit - suffix der Berechnung.
	unsigned char show;             ///< Flags ob Ausgabe auf Display erlaubt.
	unsigned char invokeInputChain; ///< Auszufuehrende Eingabekette bei Statuswechsel
	float limitLowerMessage;        ///< Wert fuer die untere Meldegrenze
	float limitLowerWarning;        ///< Wert fuer die untere Warngrenze
	float limitUpperMessage;        ///< Wert fuer die obere Meldegrenze
	float limitUpperWarning;        ///< Wert fuer die obere Warngrenze
	unsigned char sampleRate;       ///< Abtastrate fuer den analogen Eingang

	unsigned char tmp[10];          ///< Reserviert.

} TAnalogInput;
*/
typedef struct StructAnalogInput {

	unsigned char useAs;               ///< Eingangsverwendung als ...
	unsigned char show;                ///< Flags ob Ausgabe auf Display erlaubt.
	unsigned char sampleRate;          ///< Abtastrate fuer den analogen Eingang
	char name[MAX_NAME];               ///< Bezeichner der Eingaenge.
	char unit[MAX_UNIT_TXT];           ///< Einheit - suffix der Berechnung.
	float minLimit;                    ///< Minimalwert fuer Berechnung.
	float maxLimit;                    ///< Maximalwert fuer Berechnung.
	float minInput;                    ///< Minimalwert fuer Eingangsspannung.
	float maxInput;                    ///< Maximalwert fuer Eingangsspannung.
	float limitUpperAlarm;             ///< Wert fuer die obere Alarmgrenze
	float limitUpperWarning;           ///< Wert fuer die obere Warngrenze
	float limitLowerWarning;           ///< Wert fuer die untere Warngrenze
	float limitLowerAlarm;             ///< Wert fuer die untere Alarmgrenze
	unsigned char limitTolerance;      ///< Grenzwertabweichung
	unsigned char invokeInputChain[5]; ///< Auszufuehrende Eingabekette bei Statuswechsel

	unsigned char tmp[27];             ///< Reserviert.

} TAnalogInput;


///////////////////////////////////////
// Struct: StructValueGVx

/** Diese Struktur enthaelt die ggf. zu setzenden Standardwerte der globalen Variablen.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructValueGVx {
	unsigned char ucFlags;            ///< Bitfeld gibt an welche Variablen gesetzt werden sollen.
	char szValue[8][MAX_LENGTH_FELD]; ///< Zu setzender Standardwert der Variablen.
} TValueGVx;


///////////////////////////////////////
// Struct: StructFingerprint

/** Diese Struktur enthaelt die Einstellparameter fuer ein Fingerprint-Modul

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructFingerprint {
	unsigned char method;             ///< Methode des Fingerprints
	unsigned char segment;            ///< Startsegment der abgelegten Fingertemplates auf Ausweis
	unsigned char securityLevel;      ///< Sicherheitslevel; der sich aus Falschrueckweisungrate et Falschakzeptanzrate ergibt.
	float securityFRR;                ///< Falschrueckweisungsrate
	float securityFAR;                ///< Falschakzeptanzrate
	unsigned char flags;              ///< Bit(0) Templateformat => 0=DIN V66400; 1=Idencom; Bit(1-7) frei.
	unsigned char quality;            ///< Mindestbildqualitaet beim scannen
	unsigned char minutiaeCount;      ///< Mindestanzahl der Minutiae beim scannen
	unsigned char passwordGroup;      ///< Zu verwendende Passwortgruppe beim Schreiben des Ausweises
	char tmp[5];                      ///< Reserve fuer evntl. Erweiterungen
} TFingerprint;

// Zusatztypen (161 Byte)
typedef struct StructFTDIN {
	unsigned int pidHi;
	unsigned int pidLo;
	unsigned char fid;
	unsigned char quality;
	unsigned char count;
	unsigned char data[50][3];
} structFingertemplateDIN;

// Zusatztypen (216 Byte)
typedef struct StructFTIdencom {
	unsigned int pidHi;
	unsigned int pidLo;
	unsigned char fid;
	unsigned char quality;
	unsigned char count;
	unsigned char data[41][5];
} structFingertemplateIdencom;

typedef struct StructId {
	char pid[21];
	char fid[4];
} structId;

///////////////////////////////////////
// Struct: StructTBAdaption

/** Diese Struktur enthaelt die Einstellparameter fuer eine Timeboyadaption an MasterIV-Geraete

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructTBAdaption {
	unsigned char method;               ///< Methode der Adaption
	unsigned char count;                ///< Anzahl der Steckplaetze bei Mehrfachdocking
	unsigned char records;              ///< Datenablage auf MasterIV oder Timeboy
	unsigned char invokeInputChainPush; ///< Auszufuehrende Eingabekette, wenn Timeboy gesteckt wird
	unsigned char invokeInputChainPull; ///< Auszufuehrende Eingabekette, wenn Timeboy gezogen wird
	char tmp[15];                       ///< Reserve fuer evntl. Erweiterungen
} TTBAdaption;

///////////////////////////////////////
// Struct: StructGPRS

/** Diese Struktur enthaelt die Einstellparameter fuer eine GPRS-Kommunikation

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructGPRS {
	unsigned char method;               ///< Methode der GPRS-Kommunikation
	unsigned char invokeInputChain;     ///< Auszufuehrende Eingabekette, wenn Alive-Timeout abgelaufen
	char tmp[8];                        ///< Reserve fuer evntl. Erweiterungen
} TGPRS;

///////////////////////////////////////
// Struct: StructEnergieOption

/** Diese Struktur enthaelt die Einstellparameter fuer die EnergieOption zum Modbil-MasterIV.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructEnergieOption {
	unsigned char mode;                 ///< Aktivierung der Option
	unsigned char digitalIn;            ///< Eingabekette fuer das An/Aus der Zuendung
	unsigned short timeout;             ///< Timout, nach dem die Module nach einem An, eingeschaltet werden
	unsigned char invokeRelease;        ///< Eingabekette zur Fahrzeugfreigabe.
	unsigned char invokeGPS;            ///< Eingabekette bei gueltigen GPS-Daten.
	unsigned char digitalOut;           ///< Zu schaltendes Relais fuer Selbstabschaltung
	unsigned char flags;                ///< Flags fuer die Energieoption
	                                    ///< Bit0: Einzel-(0) oder Dauerfreischaltung(1).
	                                    ///< Bit1: GPS-Start/Stopp fr An/Aus der Zndung nutzen.
                                        ///< Bit2: Immer Bewegung, Erkennung fuer An/Aus der Zndung deaktiviert.
	                                    ///< Bit3-7: unused
	unsigned char powerLow;             ///< Tiefentladeschutz. Wert in Volt. 10.2V wird als 102 also *10 abgelegt
	unsigned char tmp[7];               ///< Reserve
} TEnergieOption;

///////////////////////////////////////
// Struct: StructTimer

/** Diese Struktur enthaelt die Einstellparameter fuer die Timer
	Die Struktur ist im extendet Speicherblock abgelegt.
*/
typedef struct StructTimer {
	unsigned char ucUseAs;          ///< Verwendung als Timer in Eingabeketten etc..
	unsigned short uiCycle;         ///< Bei Verwendung als Timer, dessen Zyklus.
	unsigned short uiTimeOut;       ///< Bei Verwendung als Timer, dessen Timeout.
	unsigned char invokeInputChain; ///< Auszufuehrende Eingabekette, wenn Timeout abgelaufen, oder danach der Zyklus abgelaufen.

	unsigned char tmp[10];          ///< Reserve
} TTimer;

///////////////////////////////////////
// Struct: StructGPS

/** Diese Struktur enthaelt die Einstellparameter fuer die GPS-Signalverarbeitung.
	Die Struktur ist im extendet Speicherblock abgelegt.
*/
typedef struct StructGPS
{
	unsigned char startSpeed;             ///< Zu erreichende Geschwindigkeit als Starterkennung. Einheit: km/h
	unsigned short startMove;             ///< Zu erreichende Bewegungsentfernung als alternative Starterkennung. Einheit: Meter
	unsigned char invokeInputChainStart;  ///< Auszufhrende Eingabekette fr Start Erkennung, 0 = keine Zuordnung
	unsigned short stopTimeout;           ///< Timeoutzeit fr die stopSpeed unterschritten sein muss, damit ein Stopperkennung stattfindet. Einheit: Sekunden
	unsigned char stopSpeed;              ///< Geschwindigkeit die fr erzeugen einer Stopperkennung unterschritten werden muss. Einheit: km/h
	unsigned char invokeInputChainStop;   ///< Auszufhrende Eingabekette fr Stopperkennung, 0 = keine Zuordnung
	unsigned short cycle;                 ///< Zykluszeit, in der ein Zyklusdatensatz erzeugt werden soll, wenn keine anderen Fahrdatenstze erzeugt werden. Einheit: Sekunden.
	unsigned char invokeInputChainCycle;  ///< Auszufhrende Eingabekette fr Zyklusdatensatz, 0 = keine Zuordnung
	unsigned char course;                 ///< Wert fr Kursnderung.
	unsigned char invokeInputChainCourse; ///< Auszufhrende Eingabekette fr Kursdatensatz, 0 = keine Zuordnung
	unsigned short move;                  ///< Wert fr Bewegung. Einheit: Meter.
	unsigned char invokeInputChainMove;   ///< Auszufhrende Eingabekette fr Bewegungsdatensatz, 0 = keine Zuordnung
	unsigned char tmp[2];                 ///< Reserve

} TGPS;

///////////////////////////////////////
// Struct: StructMachineDefinition

/** Diese Struktur enthaelt Informationen ueber die Definition einer Maschine.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructMachineDefinition {

	unsigned char flags;   ///< Gibt Informationsbits an. 
	                       ///< Bit 0: 0 = Deaktiviert, 1 = Aktiviert.
	char name[MAX_NAME];   ///< Bezeichnung der Maschine
	unsigned char inputs;  ///< Gibt die zugeordneten digitalen Eingnge an. 
                           ///< Bit 0: E1
                           ///< ...
                           ///< Bit 7: E8
	unsigned char ucTmp[5];///< Reserviert.

} TMachineDefinition;

///////////////////////////////////////
// Struct: StructExtended

/** Diese Struktur enthaelt zusaetzliche Informationen.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
  In dieser Struktur werden alle weiteren Informationen abgelegt.
*/
typedef struct StructExtended {

	unsigned short doubleInputTime;  ///< Zeit fuer "Doppelte Eingabe" PZE1/PZE2 Modus in Sekunden
	unsigned short confirmationTime; ///< Zeit fuer "Besttigungsfelder" in Sekunden
	// Sommer/Winterzeitumschaltung
	unsigned char daylightActive;    ///< Aktivieren der Sommer-/Winterzeitumschaltung (daylight saving)
	                                 ///< 0->Deaktiviert, 1->Alte Aktivierung, 2->Aktiv und von FW auszuwerten
	unsigned char daylight1Month;    ///< Monat in dem Umgestellt wird (1 - 12)
	unsigned char daylight1Week;     ///< Monatswoche in der Umgestellt wird (1 - 5, wobei 5 letzte im Monat bedeutet)
	unsigned char daylight1Day;      ///< Tag der Umschaltung (1 - 7, wobei 1 = Montag)
	unsigned char daylight1Hour;     ///< Stunde fuer Umschaltung (0 - 23)
	unsigned char daylight2Month;    ///< Monat in dem Umgestellt wird (1 - 12)
	unsigned char daylight2Week;     ///< Monatswoche in der Umgestellt wird (1 - 5, wobei 5 letzte im Monat bedeutet)
	unsigned char daylight2Day;      ///< Tag der Umschaltung (1 - 7, wobei 1 = Montag)
	unsigned char daylight2Hour;     ///< Stunde fuer Umschaltung (0 - 23)
	char transponderPassword32[9];   ///< Passwort fuer Hitag2 (32bit) in ASCII (Hex) abgelegt.
	char transponderPasswordA0[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
	unsigned char phgAESKey[16];     ///< Schluessel fuer Kommunikationsverschluesselung der PHG Leser
	TFingerprint fingerprint;        ///< Datenstruktur zum Fingerprint
	TTBAdaption adaption;            ///< Anbindungsstruktur von Timboys an MasterIV
	TGPRS gprs;                      ///< Struktur fuer Zusatzangaben bezueglich GPRS, wie z. B. Alive.
	TEnergieOption energieOption;    ///< Struktur fuer Energieoption bei Mobil-MasterIV
	TTimer timer[4];				 ///< Struktur fuer Timer in der Signalverarbeitung (vorgesehen sind 4 Timer a 16 Byte)
	unsigned char transponderPasswordFlag; ///< Passwortflag fuer Passwortverwendung von Mifare
		                                   ///< Bit 0, 1: Passwortverwendung fr Lesen/Schreiben A0/B0; 0=KeyA/KeyA, 1=KeyA/KeyB, 2=KeyB/KeyB
                                           ///< Bit 2, 3: Passwortverwendung fr Lesen/Schreiben A1/B1; 0=KeyA/KeyA, 1=KeyA/KeyB, 2=KeyB/KeyB
                                           ///< Bit 4, 5: Passwortverwendung fr Lesen/Schreiben A2/B2; 0=KeyA/KeyA, 1=KeyA/KeyB, 2=KeyB/KeyB
	                                       ///< Bit 6: Frei
	                                       ///< Bit 7: Autologin; Verwendung aller Passwrter fr Lesen
	char transponderPasswordB0[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
	char transponderPasswordA1[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
	char transponderPasswordB1[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
	char transponderPasswordA2[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
	char transponderPasswordB2[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
	int transponderSectorTrailer;    ///< Zugriffseinstellungen fr den globale Sektor.
	unsigned char transponderInitialValue[17]; ///< Startwert von Mifare-Ausweis
	unsigned char memoryUsage;           ///< Speicheraufteilung zwischen Daten/Listen in %. Wert gibt die % fr Daten reserviert an.
	unsigned char entranceOnline;        ///< Betriebsmodus der Zutrittskontrolle. 0=Offline, 1=Online, 2=Online/Offline mit Timeout.
	unsigned char entranceOnlineTimeout; ///< Timeout fuer die Umschaltung zwischen Online/Offline Betrieb der ZK.
	TDigitalInputExt digitalExtE[8];     ///< Erweiterte Strukturen fr die Digitalen Eingaenge.
	char fileName[44];                   ///< Dateiname der Setupdatei. Wird vor der bertragung durch Laderoutine der Setupdatei gesetzt.
	char fileTimeStamp[20];              ///< Modifizierungszeitstempel der Setupdatei. Wird vor der bertragung durch Laderoutine der Setupdatei gesetzt.
	unsigned char communicationPassword[17]; ///< Passwort fr die Kommunikation
	TGPS gps;							 ///< Struktur fr GPS Datensatz Erzeugung
	unsigned char userPasswordEnable;    ///< Benutzerpasswort verwenden, ansonsten wird das Systempasswort frs Userpasswort verwendet.
	unsigned char escJumpToMainmenu;     ///< Im PZE-Modus wird durch drcken der ESC-Taste das Hauptmen angezeigt.
	unsigned char ledPerFieldFunction;   ///< AE/BDE-MasterIV: LED steuerung mit Feldfunktion "LED schalten" steuern.
	unsigned short dataWriteOKTime;      ///< Die Anzeigezeit der Meldung "Datenspeicherung erfolgreich". 0=aus, 500ms default,bereich 0,500-5000 in 250ms Schritten; Einheit ms
	unsigned char dataWriteOKFontType;   ///< Der verwendete Zeichensatztyp bei Meldung "Datenspeicherung erfolgreich". 0=
	unsigned int powerActiv;            ///< Gibt an ob die Energieverwaltung des Setups aktiviert ist, inclusive Flags fr die aktivierten Module.
	                                     ///< siehe PAF_x (P)ower(A)ctiv(F)lag
	unsigned char powerClockTimerCount;  ///< Gibt an wie viele Timer die zu einer Bestimmten Uhrzeit auslsen existieren drfen.
	unsigned char spsQuery;              ///< Abfrage einer angeschlossenen SPS. Feldfunktion SPS-Abfrage.
	STransponderLegicAdvant legicAdvant; ///< Transponderwerte fuer Legic Advant
	char transponderPasswordA0_enhanced[21];  ///< Passwort fuer Mifare zusaetzlich (80bit) in ASCII (Hex) abgelegt.
	char transponderPasswordB0_enhanced[21];  ///< Passwort fuer Mifare zusaetzlich (80bit) in ASCII (Hex) abgelegt.
	char transponderPasswordA1_enhanced[21];  ///< Passwort fuer Mifare zusaetzlich (80bit) in ASCII (Hex) abgelegt.
	char transponderPasswordB1_enhanced[21];  ///< Passwort fuer Mifare zusaetzlich (80bit) in ASCII (Hex) abgelegt.
	char transponderPasswordA2_enhanced[21];  ///< Passwort fuer Mifare zusaetzlich (80bit) in ASCII (Hex) abgelegt.
	char transponderPasswordB2_enhanced[21];  ///< Passwort fuer Mifare zusaetzlich (80bit) in ASCII (Hex) abgelegt.
	unsigned char tmp[318];              ///< Frei fuer Erweiterungen
} TExtended;


// Defines und Strukturen fr die Energieschemen
#include "Power.h"

#endif
